﻿#define NOTATEST


using IndustrialNetworks.ModbusCore.ASCII;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Timers;
using System.IO.Ports;
using NModbus;
using NModbus.Serial;


namespace ContSys
{
    public class StateMachine
    {

        /*
         * CONSTANTS (MODBUS-addresses on the PLC)
         */


        const int ADDR_START = 0000;

        const int ADDR_TEMPERATURE_SENSOR1 = 8208;
        const int ADDR_TEMPERATURE_SENSOR2 = 8208;
        const int ADDR_TEMPERATURE_SENSOR3 = 8208;
        const int ADDR_TEMPERATURE_SENSOR4 = 8208;
        const int ADDR_TEMPERATURE_SENSOR5 = 8208;

        const int ADDR_PRESSURE_SENSOR1 = 0000;
        const int ADDR_PRESSURE_SENSOR2 = 0000;

        const int ADDR_COMPRESSOR_STATUS = 1024;
        const int ADDR_COMPRESSOR_AUTO_MAN = 2049;
        const int ADDR_SET_COMPRESSOR = 1280;

        const int ADDR_COOLING_STATUS = 1025;
        const int ADDR_COOLING_AUTO_MAN = 2050;
        const int ADDR_SET_COOLING = 1281;






        /*
         * sensor values to be read from plc
         */
        //static StateMachine instance;



       
        float Temperature_sensor1 = -999;
        float Temperature_sensor2 = -999;
        float Temperature_sensor3 = -999;
        float Temperature_sensor4 = -999;
        float Temperature_sensor5 = -999;

        float Pressure_sensor1 = -999;
        float Pressure_sensor2 = -999;

        bool Compressor_Status = false;
        bool Cooling_Status = false;

        bool PLC_connect = false;

        private System.Timers.Timer timer;
        FileLogger filelogger;

        private ModbusASCIIMaster objModbusMaster = null;
        private IModbusMaster nModbusMaster = null;
        private IModbusMaster gModbusMaster = null;

        /*
         * configuration settings : connection and modes
         */
        private bool rtu_or_asci = true;    //true->rtu false->asci
        private string slavePort = "COM2";
        bool control_or_monitoringOnly = false;     //true->control false->only monitoring
        bool automatic_or_manual = false;     //true->automatic false->manual (not implemented yet)



        //public static StateMachine getInstance()
        //{
        //    return instance;
        //}

        //public StateMachine()
        //{
        //    instance = this;
        //}

        public void Start()
        {
            filelogger = new FileLogger(this);

            try
            {
                if (rtu_or_asci)
                {
                    //init NModbusMaster
                    SerialPort port = new SerialPort(slavePort, 9600, Parity.None, 8, StopBits.One);
                    port.Open();

                    nModbusMaster = (new ModbusFactory()).CreateRtuMaster(new SerialPortAdapter(port));
                    gModbusMaster = nModbusMaster;
                }
                else
                {
                    objModbusMaster = new ModbusASCIIMaster(slavePort, 9600, 7, System.IO.Ports.StopBits.One, System.IO.Ports.Parity.Even);
                    //Console.WriteLine("try COM CONNECTIONNNNNNNNNNNN!!!!!!!!!!!!!!!!!!!");
                    objModbusMaster.Connection();
                    gModbusMaster = (IModbusMaster)objModbusMaster;
                }
                Console.WriteLine("CONNECTEDDDDDDDDDDDDDDDDDD!!!!!!!!!!!!!!!!!!!");
                PLC_connect = true;


                SetTimer();

            }
            catch(Exception ex)
            {
                Console.WriteLine("COM CONNECTION FAILEDDDDDDDDDDDDDDDDDDDDDD!!!!!!!!!!!!!!!!!!!");
                Console.WriteLine(ex.Message);
                PLC_connect = false;
            }


        }

        private void SetTimer()
        {
            // Create a timer with a two second interval.
            timer = new System.Timers.Timer(1000);
            // Hook up the Elapsed event for the timer. 
            timer.Elapsed += OnTimedEvent;
            timer.AutoReset = true;
            timer.Enabled = true;
        }

        private void OnTimedEvent(Object source, ElapsedEventArgs e)
        {
            Console.WriteLine("check Sensors at {0:HH:mm:ss.fff}",
                              e.SignalTime);
            CheckSensors();
            filelogger.log();
        }

        private void CheckSensors()
        {
            String currentReading = "";
            try
            {


                Temperature_sensor1 = gModbusMaster.ReadHoldingRegisters(4, ADDR_TEMPERATURE_SENSOR1, 1)[0];
               /*
                
                Temperature_sensor2 = gModbusMaster.ReadHoldingRegisters(5, ADDR_TEMPERATURE_SENSOR2, 1)[0];
               Temperature_sensor3 = gModbusMaster.ReadHoldingRegisters(6, ADDR_TEMPERATURE_SENSOR3, 1)[0];
               Temperature_sensor4 = gModbusMaster.ReadHoldingRegisters(7, ADDR_TEMPERATURE_SENSOR4, 1)[0];
               Temperature_sensor5 = gModbusMaster.ReadHoldingRegisters(8, ADDR_TEMPERATURE_SENSOR5, 1)[0];


                Pressure_sensor1 = gModbusMaster.ReadHoldingRegisters(1, ADDR_PRESSURE_SENSOR1, 1)[0];
                Pressure_sensor2 = gModbusMaster.ReadHoldingRegisters(1, ADDR_PRESSURE_SENSOR2, 1)[0];
                Compressor_Status = gModbusMaster.ReadCoils(1, ADDR_COMPRESSOR_STATUS, 1)[0];
                Cooling_Status = gModbusMaster.ReadCoils(1, ADDR_COOLING_STATUS, 1)[0];
               */



            }
            catch (Exception ex)
            {
                Console.WriteLine(currentReading+": READ ERROR!!!!!!!!!!!!!!!!!!!");
                Console.WriteLine(ex.Message);
            }
        }

        /*
        *
        * write in AUCTUATORS register FUNCTIONS
        *
        */

        /*

        public void START_SYSTEM(bool onoff)
        {
#if NOTATEST
            gModbusMaster.WriteSingleCoil(1, ADDR_START, onoff);
#endif
        }
        public void COMPRESSOR_AUTO_MAN(bool onoff)
        {
#if NOTATEST
            gModbusMaster.WriteSingleCoil(2, ADDR_COMPRESSOR_AUTO_MAN, onoff);
#endif
        }


        public void SET_COMPRESSOR_OnOff(bool onoff)
        {
     #if NOTATEST
            gModbusMaster.WriteSingleCoil(2, ADDR_SET_COMPRESSOR, onoff);
     #endif
        }


        public void COOLING_AUTO_MAN(bool onoff)
        {
#if NOTATEST
            gModbusMaster.WriteSingleCoil(2, ADDR_COOLING_AUTO_MAN, onoff);
#endif
        }


        public void SET_COOLING_OnOff(bool onoff)
        {
#if NOTATEST
            gModbusMaster.WriteSingleCoil(2, ADDR_SET_COOLING, onoff);
#endif
        }

        /*
        *
        * Getter/Setter
        *
        */

        public float TEMPERATURE_SENSOR1 { get => Temperature_sensor1 / 10; }
        public float TEMPERATURE_SENSOR2 { get => Temperature_sensor2 / 10; }
        public float TEMPERATURE_SENSOR3 { get => Temperature_sensor3 / 10; }
        public float TEMPERATURE_SENSOR4 { get => Temperature_sensor4 / 10; }
        public float TEMPERATURE_SENSOR5 { get => Temperature_sensor5 / 10; }

        public float PRESSURE_SENSOR1 { get => Pressure_sensor1 ; }
        public float PRESSURE_SENSOR2 { get => Pressure_sensor2; }
        public bool COMPRESSOR_STATUS { get => Compressor_Status; }
        public bool COOLING_STATUS { get => Cooling_Status; }


         public bool PLC_Connect { get => PLC_connect; }
        
        //for configurations
        public string SlavePort { get => slavePort; set => slavePort = value; }
    }
}
